@@ -26,6 +26,8 @@
 
 #include <string.h>
 
+static GObjectClass *parent_class;
+
 struct _GsfInputTextline {
 	GsfInput input;
 
@@ -72,7 +74,6 @@ gsf_input_textline_new (GsfInput *source)
 static void
 gsf_input_textline_finalize (GObject *obj)
 {
-	GObjectClass *parent_class;
 	GsfInputTextline *input = (GsfInputTextline *)obj;
 
 	if (input->source != NULL) {
@@ -85,20 +86,18 @@ gsf_input_textline_finalize (GObject *obj)
 	}
 	input->buf_size = 0;
 
-	parent_class = g_type_class_peek (GSF_INPUT_TYPE);
-	if (parent_class && parent_class->finalize)
-		parent_class->finalize (obj);
+	parent_class->finalize (obj);
 }
 
 static GsfInput *
-gsf_input_textline_dup (GsfInput *src_input, GError **err)
+gsf_input_textline_dup (GsfInput *src_input, G_GNUC_UNUSED GError **err)
 {
 	GsfInputTextline const *src = (GsfInputTextline *)src_input;
 	GsfInputTextline *dst = g_object_new (GSF_INPUT_TEXTLINE_TYPE, NULL);
 
-	(void)err;
 	dst->source = src->source;
 	g_object_ref (G_OBJECT (dst->source));
+	gsf_input_set_size (GSF_INPUT (dst), gsf_input_size (src_input));
 
 	return GSF_INPUT (dst);
 }
@@ -141,6 +140,8 @@ gsf_input_textline_class_init (GObjectClass *gobject_class)
 	input_class->Dup	= gsf_input_textline_dup;
 	input_class->Read	= gsf_input_textline_read;
 	input_class->Seek	= gsf_input_textline_seek;
+
+	parent_class = g_type_class_peek_parent (gobject_class);
 }
 
 GSF_CLASS (GsfInputTextline, gsf_input_textline,
@@ -151,7 +152,7 @@ GSF_CLASS (GsfInputTextline, gsf_input_textline,
  * @input :
  *
  * A utility routine to read things line by line from the underlying source.
- * Trailing newlines and carriage returns are stipped, and the resultant buffer
+ * Trailing newlines and carriage returns are stripped, and the resultant buffer
  * can be edited.
  *
  * returns the string read, or NULL on eof.
@@ -185,8 +186,7 @@ gsf_input_textline_ascii_gets (GsfInputTextline *textline)
 
 		/* copy the remains into the buffer, grow it if necessary */
 		len = ptr - textline->remainder;
-		/* "+1", just in case len==0.  */
-		if (count + len + 1 > textline->buf_size) {
+		if (count + len >= textline->buf_size) {
 			textline->buf_size += len;
 			textline->buf = g_renew (guint8, textline->buf,
 						 textline->buf_size + 1);
@@ -239,7 +239,7 @@ gsf_input_textline_ascii_gets (GsfInputTextline *textline)
  * @input :
  *
  * A utility routine to read things line by line from the underlying source.
- * Trailing newlines and carriage returns are stipped, and the resultant buffer
+ * Trailing newlines and carriage returns are stripped, and the resultant buffer
  * can be edited.
  *
  * returns the string read, or NULL on eof.
@@ -273,7 +273,7 @@ gsf_input_textline_utf8_gets (GsfInputTextline *textline)
 
 		/* copy the remains into the buffer, grow it if necessary */
 		len = ptr - textline->remainder;
-		if ((count + len) > textline->buf_size) {
+		if (count + len >= textline->buf_size) {
 			textline->buf_size += len;
 			textline->buf = g_renew (guint8, textline->buf,
 						 textline->buf_size + 1);
